Library myAVRMK2;

// myAVR MK2 Driver for WinFACT (all versions)
// DELHPI 2009
// AVR Studio 4
//
// ===== Commands:
//
// ----- Write Output Pin
//
// Step 1: BORIS sends 4-Byte-Command to MK2
//
// Command structure: Wxyz
//
// x: 'B', 'C', 'D'  Port to write (char)
// y: 0 ... 7    Pin to write (byte)
// z: Pin Mode (char)
//    'O': Digital Output
//    'P': PWM Output
//
// Step 2: MK2 returns 'OK'
//
// Step 3: BORIS sends value to write (byte):
//          0 or 1 for Digital Output
//          0 ... 255 for PWM Output
//
// Step 4: MK2 returns 'OK'
//

// ----- Read Input Pin
//
// Step 1: BORIS sends 4-Byte-Command to MK2
//
// Command structure: Rxyz
//
// x: 'B', 'C', 'D'  Port to read (char)
// y: 0 ... 7    Pin to read (byte)
// z: Pin Mode
//    'I': Digital Input
//    'A': Analog Input
//
// Step 2:
//
// MK2 returns byte value 0 or 1 for Digital Input
// MK2 returns 2-byte value 0 ... 1023 for Analog Input (Low Byte first)
//

{$R *.res}

uses
  WinProcs,
  WinTypes,
  SysUtils,
  Controls,
  Forms,
  Classes,
  Graphics,
  StdCtrls,
  Dialogs,
  Math,
  myAVRMK2Dlg in 'myAVRMK2Dlg.pas' {MK2Dialog};

type
  TPinMode = (UNUSED, DIGIN, DIGOUT, ANAIN, PWMOUT);

  PUserData = ^TUserData;
  TUserData = record
    ReceivedString: string; // Received String
    // Configuration data
    COMPort: integer;     // COM-Port
    Baudrate: integer;    // Baud rate for COM-Port
    PBMode: array[0..5] of TPinMode; // Port B
    PCMode: array[0..5] of TPinMode; // Port C
    PDMode: array[2..7] of TPinMode; // Port D
  end;

  PParameterStruct=^TParameterStruct;
  TParameterStruct=packed record
    NuE : Byte;
    NuI : Byte;
    NuB : Byte;
    E:Array[0..31] of Extended;
    I:Array[0..31] of Integer;
    B:Array[0..31] of Byte;
    D:Array[0..255] of AnsiChar;
    EMin:Array[0..31] of Extended;
    EMax:Array[0..31] of Extended;
    IMin:Array[0..31] of Integer;
    IMax:Array[0..31] of Integer;
    NaE : Array[0..31,0..40] of AnsiChar;
    NaI : Array[0..31,0..40] of AnsiChar;
    NaB : Array[0..31,0..40] of AnsiChar;
    UserDataPtr: PUserData;
    ParentPtr: Pointer;
    ParentHWnd: HWnd;
    ParentName: PAnsiChar;
    UserHWindow: HWnd;
    DataFile: text;
  end;

  PDialogEnableStruct=^TDialogEnableStruct;
  TDialogEnablestruct=packed record
    AllowE: Longint;
    AllowI: Longint;
    AllowB: Longint;
    AllowD: Byte;
  end;

  PNumberOfInputsOutputs=^TNumberOfInputsOutputs;
  TNumberOfInputsOutputs=packed record
    Inputs :Byte;
    Outputs:Byte;
    NameI : Array[0..49,0..40] of AnsiChar;
    NameO : Array[0..49,0..40] of AnsiChar;
  end;

  PInputArray = ^TInputArray;
  TInputArray = packed array[1..30] of extended;
  POutputArray = ^TOutputArray;
  TOutputArray = packed array[1..30] of extended;

var
  COMHandle: THandle;

procedure GetParameterStruct(D:PParameterStruct);export stdcall;
begin
  D^.NuE := 0;
  D^.NuI := 0;
  D^.NuB := 0;
  D^.I[0] := 0;
end;

procedure GetDialogEnableStruct(D:PDialogEnableStruct;D2:PParameterStruct);export stdcall;
begin
  D^.AllowE:=$FFFFFFFF;
  D^.AllowB:=$FFFFFFFF;
  D^.AllowI:=$FFFFFFFF;
  D^.AllowD:= 1;
end;

procedure GetNumberOfInputsOutputs(D:PNumberOfInputsOutputs);export stdcall;
var i: integer;
begin
  D^.Outputs := 18;
  D^.Inputs := 18;
  // Outputs
  StrPCopy(D^.NameO[0], 'PB0');
  StrPCopy(D^.NameO[1], 'PB1');
  StrPCopy(D^.NameO[2], 'PB2');
  StrPCopy(D^.NameO[3], 'PB3');
  StrPCopy(D^.NameO[4], 'PB4');
  StrPCopy(D^.NameO[5], 'PB5');
  StrPCopy(D^.NameO[6], 'PC0');
  StrPCopy(D^.NameO[7], 'PC1');
  StrPCopy(D^.NameO[8], 'PC2');
  StrPCopy(D^.NameO[9], 'PC3');
  StrPCopy(D^.NameO[10], 'PC4');
  StrPCopy(D^.NameO[11], 'PC5');
  StrPCopy(D^.NameO[12], 'PD2');
  StrPCopy(D^.NameO[13], 'PD3');
  StrPCopy(D^.NameO[14], 'PD4');
  StrPCopy(D^.NameO[15], 'PD5');
  StrPCopy(D^.NameO[16], 'PD6');
  StrPCopy(D^.NameO[17], 'PD7');
  // Inputs
  for i:=1 to 18 do
    StrCopy(D^.NameI[i-1], D^.NameO[i-1]);
end;

function SetOutputChar: PAnsiChar; export stdcall;
begin
  Result := 'B&0B&1B&2B&3B&4B&5C&0C&1C&2C&3C&4C&5D&2D&3D&4D&5D&6D&7';
end;

function SetInputChar: PAnsiChar; export stdcall;
begin
  Result := 'B&0B&1B&2B&3B&4B&5C&0C&1C&2C&3C&4C&5D&2D&3D&4D&5D&6D&7';
end;

function ReadByte: byte;
// Read Byte
var i, StartTime: integer;
    BytesRead: DWORD;
    InBytes: array[1..1] of byte;
begin
  StartTime := GetTickCount;
  repeat
    InBytes[1] := 0;
    ReadFile(ComHandle, InBytes, 1, BytesRead, nil);
    if BytesRead > 0 then
      Result := InBytes[1];
    Application.ProcessMessages;
  until (BytesRead > 0) or ((GetTickCount - StartTime) > 1000);
end;

procedure WaitForOK;
begin
  ReadByte;
  ReadByte;
end;

procedure SendString(Command: AnsiString);
// Send String
var Buffer: array[0..100] of AnsiChar;
    BytesWritten: DWORD;
    i: integer;
begin
  for i:=1 to Length(Command) do
    WriteFile(ComHandle, Command[i], 1, BytesWritten, nil);
end;

function CanSimulateDLL(D:PParameterStruct):Integer; export stdcall;
// Called to check if simulation is possible
var ComHandle: THandle;
begin
  Screen.Cursor := crHourGlass;
  Result := 0;
  ComHandle := CreateFile(PChar('COM' + IntToStr(D^.UserDataPtr.ComPort)),GENERIC_READ or GENERIC_WRITE,0,nil,OPEN_EXISTING,0,0);
  if(ComHandle <> INVALID_HANDLE_VALUE) then begin
    Result := 1;
    CloseHandle(ComHandle);
    ComHandle := 0;
  end;
  Screen.Cursor := crDefault;
end;

procedure SimulateDLL(T:Extended;D1:PParameterStruct;Inputs:PInputArray;Outputs:POutputArray);export stdcall;
// Called for each simulation step
var
  i: integer;
  s: string;
  b: byte;
function WriteString(Port: string; Pin: integer; PinMode: TPinMode): AnsiString;
begin
  Result := 'W' + Port + AnsiChar(Pin);
  case PinMode of
    DIGOUT: Result := Result + 'O';
    PWMOUT: Result := Result + 'P';
  end;
end;
function ReadString(Port: string; Pin: integer; PinMode: TPinMode): AnsiString;
begin
  Result := 'R' + Port + AnsiChar(Pin);
  case PinMode of
    DIGIN: Result := Result + 'I';
    ANAIN: Result := Result + 'A';
  end;
end;
begin
  with D1^.UserDataPtr^ do begin
    // Port B
    for i:=0 to 5 do begin
      case PBMode[i] of
        DIGIN:  begin
                  SendString(ReadString('B', i, PBMode[i]));
                  Outputs^[i+1] := 5*ReadByte;
                end;
        DIGOUT: begin
                  SendString(WriteString('B', i, PBMode[i]));
                  WaitForOK;
                  SendString(AnsiChar(ord(Inputs^[i+1] > 2.5)));
                  WaitForOK;
                end;
        PWMOUT: begin
                  SendString(WriteString('B', i, PBMode[i]));
                  WaitForOK;
                  SendString(AnsiChar(round(Min(Max(Inputs^[i+1], 0), 255))));
                  WaitForOK;
                end;
      end;
    end;
    // Port C
    for i:=0 to 5 do begin
      case PCMode[i] of
        DIGIN:  begin
                  SendString(ReadString('C', i, PCMode[i]));
                  Outputs^[i+6+1] := 5*ReadByte;
                end;
        ANAIN:  begin
                  SendString(ReadString('C', i, PCMode[i]));
                  Outputs^[i+6+1] := ReadByte + 256*ReadByte;
                end;
        DIGOUT: begin
                  SendString(WriteString('C', i, PCMode[i]));
                  WaitForOK;
                  SendString(AnsiChar(ord(Inputs^[i+6+1] > 2.5)));
                  WaitForOK;
                end;
      end;
    end;
    // Port D
    for i:=2 to 7 do begin
      case PDMode[i] of
        DIGIN:  begin
                  SendString(ReadString('D', i, PDMode[i]));
                  Outputs^[i-2+6+6+1] := 5*ReadByte;
                end;
        DIGOUT: begin
                  SendString(WriteString('D', i, PDMode[i]));
                  WaitForOK;
                  SendString(AnsiChar(ord(Inputs^[i-2+6+6+1] > 2.5)));
                  WaitForOK;
                end;
      end;
    end;
  end;
end;

procedure InitSimulationDLL(D:PParameterStruct;Inputs:PInputArray;Outputs:POutputArray);export stdcall;
// Called when simulation starts
var DCB: TDCB;
    CommTimeOuts: TCommTimeOuts;
    s: string;
    i: integer;
begin
  if ComHandle = 0 then begin
    ComHandle := CreateFile(PChar('COM' + IntToStr(D^.UserDataPtr.ComPort)),GENERIC_READ or GENERIC_WRITE,0,nil,OPEN_EXISTING,0,0);
    DCB.DCBlength := SizeOf(DCB);
    GetCommState(ComHandle, DCB);
    DCB.BaudRate := D^.UserDataPtr.Baudrate;
    DCB.ByteSize := 8;
    DCB.Parity := 0;
    DCB.StopBits := ONESTOPBIT;
    DCB.Flags := 1;
    SetCommState(ComHandle, DCB);
    GetCommTimeOuts(ComHandle, CommTimeOuts);
    CommTimeOuts.ReadTotalTimeoutConstant := 10;
    SetCommTimeOuts(ComHandle, CommTimeOuts);
    sleep(100);
  end;
  SimulateDLL(0, D, Inputs, Outputs);
end;

procedure EndSimulationDLL2(D:PParameterStruct); export stdcall;
// Called when simulation was terminated
begin
  if ComHandle <> 0 then
    CloseHandle(ComHandle);
  ComHandle := 0;
end;

procedure InitUserDLL(D: PParameterStruct); export stdcall;
var i: integer;
begin
  Application.Handle := D^.ParentHWnd;
  D^.UserDataPtr := new(PUserData);
  with D^.UserDataPtr^ do begin
    COMPort := 1;
    Baudrate := 9600;
    PBMode[0] := DIGOUT;
    for i:=1 to 3 do
      PBMode[i] := PWMOUT;
    for i:=4 to 5 do
      PBMode[i] := DIGOUT;
    for i:=0 to 5 do
      PCMode[i] := ANAIN;
    for i:=2 to 7 do
      PDMode[i] := DIGIN;
  end;
end;

procedure DisposeUserDLL(D: PParameterStruct); export stdcall;
begin
  Application.Handle := 0;
  Dispose(D^.UserDataPtr);
end;

function GetDLLName: PAnsiChar; export stdcall;
begin
  GetDLLName := 'myAVR MK2';
end;

procedure CallParameterDialogDLL(D1:PParameterStruct; D2:PNumberOfInputsOutputs);export stdcall;
var Dialog: TMK2Dialog;
begin
  Dialog := TMK2Dialog.Create(Application);
  with Dialog, D1^.UserDataPtr^ do begin
    UpDown1.Position := COMPort;
    case Baudrate of
      1200:   BaudCombobox.ItemIndex := 0;
      2400:   BaudCombobox.ItemIndex := 1;
      4800:   BaudCombobox.ItemIndex := 2;
      9600:   BaudCombobox.ItemIndex := 3;
      19200:  BaudCombobox.ItemIndex := 4;
      38400:  BaudCombobox.ItemIndex := 5;
      115200: BaudCombobox.ItemIndex := 6;
    end;
    PB0CB.ItemIndex := ord(PBMode[0]);
    if PBMode[1] = PWMOUT then
      PB1CB.ItemIndex := 3
    else
      PB1CB.ItemIndex := ord(PBMode[1]);
    if PBMode[2] = PWMOUT then
      PB2CB.ItemIndex := 3
    else
      PB2CB.ItemIndex := ord(PBMode[2]);
    if PBMode[3] = PWMOUT then
      PB3CB.ItemIndex := 3
    else
      PB3CB.ItemIndex := ord(PBMode[3]);
    PB4CB.ItemIndex := ord(PBMode[4]);
    PB5CB.ItemIndex := ord(PBMode[5]);
    PC0CB.ItemIndex := ord(PCMode[0]);
    PC1CB.ItemIndex := ord(PCMode[1]);
    PC2CB.ItemIndex := ord(PCMode[2]);
    PC3CB.ItemIndex := ord(PCMode[3]);
    PC4CB.ItemIndex := ord(PCMode[4]);
    PC5CB.ItemIndex := ord(PCMode[5]);
    PD2CB.ItemIndex := ord(PDMode[2]);
    PD3CB.ItemIndex := ord(PDMode[3]);
    PD4CB.ItemIndex := ord(PDMode[4]);
    PD5CB.ItemIndex := ord(PDMode[5]);
    PD6CB.ItemIndex := ord(PDMode[6]);
    PD7CB.ItemIndex := ord(PDMode[7]);
    if ShowModal = mrOK then begin
      COMPort := UpDown1.Position;
      case BaudCombobox.ItemIndex of
        0: Baudrate := 1200;
        1: Baudrate := 2400;
        2: Baudrate := 4800;
        3: Baudrate := 9600;
        4: Baudrate := 19200;
        5: Baudrate := 38400;
        6: Baudrate := 115200;
      end;
      PBMode[0] := TPinMode(PB0CB.ItemIndex);
      if PB1CB.ItemIndex = 3 then
        PBMode[1] := PWMOUT
      else
        PBMode[1] := TPinMode(PB1CB.ItemIndex);
      if PB2CB.ItemIndex = 3 then
        PBMode[2] := PWMOUT
      else
        PBMode[2] := TPinMode(PB2CB.ItemIndex);
      if PB3CB.ItemIndex = 3 then
        PBMode[3] := PWMOUT
      else
        PBMode[3] := TPinMode(PB3CB.ItemIndex);
      PBMode[4] := TPinMode(PB4CB.ItemIndex);
      PBMode[5] := TPinMode(PB5CB.ItemIndex);
      PCMode[0] := TPinMode(PC0CB.ItemIndex);
      PCMode[1] := TPinMode(PC1CB.ItemIndex);
      PCMode[2] := TPinMode(PC2CB.ItemIndex);
      PCMode[3] := TPinMode(PC3CB.ItemIndex);
      PCMode[4] := TPinMode(PC4CB.ItemIndex);
      PCMode[5] := TPinMode(PC5CB.ItemIndex);
      PDMode[2] := TPinMode(PD2CB.ItemIndex);
      PDMode[3] := TPinMode(PD3CB.ItemIndex);
      PDMode[4] := TPinMode(PD4CB.ItemIndex);
      PDMode[5] := TPinMode(PD5CB.ItemIndex);
      PDMode[6] := TPinMode(PD6CB.ItemIndex);
      PDMode[7] := TPinMode(PD7CB.ItemIndex);
    end;
  end;
  Dialog.Free;
end;

Procedure WriteToFile(AFileHandle:Word; D: PParameterStruct); export stdcall;
var i: integer;
    s: array[0..1000] of AnsiChar;
begin
  with D^.UserDataPtr^ do begin
    StrPCopy(s, IntToStr(ComPort) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    StrPCopy(s, IntToStr(Baudrate) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    for i:=0 to 5 do begin
      StrPCopy(s, IntToStr(ord(PBMode[i])) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    end;
    for i:=0 to 5 do begin
      StrPCopy(s, IntToStr(ord(PCMode[i])) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    end;
    for i:=2 to 7 do begin
      StrPCopy(s, IntToStr(ord(PDMode[i])) + #13#10); _lWrite(AFileHandle, s, StrLen(s));
    end;
  end;
end;

Procedure ReadFromFile(AFileHandle:Word; D: PParameterStruct); export stdcall;
var i: integer;
    s: array[0..1000] of AnsiChar;
procedure ReadOneLine(FHandle:Word; Aps :PAnsiChar);
var i  : Integer;
begin
  i:=0;
  _lRead(FHandle,@Aps[i],1);
  repeat
    inc(i);
    _lRead(FHandle,@Aps[i],1);
  until (Aps[i-1]=#13) and (Aps[i]=#10);
  Aps[i-1]:=#0;
end;
begin
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.COMPort := StrToInt(StrPas(s));
  ReadOneLine(AFileHandle, s); D^.UserDataPtr.Baudrate := StrToInt(StrPas(s));
  for i:=0 to 5 do begin
    ReadOneLine(AFileHandle, s); D^.UserDataPtr.PBMode[i] := TPinMode(StrToInt(StrPas(s)));
  end;
  for i:=0 to 5 do begin
    ReadOneLine(AFileHandle, s); D^.UserDataPtr.PCMode[i] := TPinMode(StrToInt(StrPas(s)));
  end;
  for i:=2 to 7 do begin
    ReadOneLine(AFileHandle, s); D^.UserDataPtr.PDMode[i] := TPinMode(StrToInt(StrPas(s)));
  end;
end;

procedure IsUserDLL32; export stdcall;
begin
end;

procedure IsDemoDLL; export stdcall;
begin
end;

exports
  GetParameterStruct,
  GetDialogEnableStruct,
  GetNumberOfInputsOutputs,
  CanSimulateDLL,
  InitSimulationDLL,
  SimulateDLL,
  InitUserDLL,
  DisposeUserDLL,
  EndSimulationDLL2,
  GetDLLName,
  CallParameterDialogDLL,
  WriteToFile,
  ReadFromFile,
  SetInputChar,
  SetOutputChar,
  IsDemoDLL,
  IsUserDLL32;


begin
  DecimalSeparator := '.';
  Application.UpdateFormatSettings := false;
  COMHandle := 0;
end.
